@@ -1,7 +1,8 @@
 import urllib
 from oauth2 import OAuth2Request
 import re
-import simplejson
+from json_import import simplejson
+
 re_path_template = re.compile('{\w+}')
 
 
@@ -11,11 +12,15 @@ def encode_string(value):
 
 
 class InstagramClientError(Exception):
-    def __init__(self, error_message):
+    def __init__(self, error_message, status_code=None):
+        self.status_code = status_code
         self.error_message = error_message
 
     def __str__(self):
-        return self.error_message
+        if self.status_code:
+            return "(%s) %s" % (self.status_code, self.error_message)
+        else:
+            return self.error_message
 
 
 class InstagramAPIError(Exception):
@@ -46,8 +51,13 @@ def bind_method(**config):
         def __init__(self, api, *args, **kwargs):
             self.api = api
             self.as_generator = kwargs.pop("as_generator", False)
+            if self.as_generator:
+                self.pagination_format = 'next_url'
+            else:
+                self.pagination_format = kwargs.pop('pagination_format', 'next_url')
             self.return_json = kwargs.pop("return_json", False)
             self.max_pages = kwargs.pop("max_pages", 3)
+            self.with_next_url = kwargs.pop("with_next_url", None)
             self.parameters = {}
             self._build_parameters(args, kwargs)
             self._build_path()
@@ -86,14 +96,37 @@ def bind_method(**config):
                 self.path = self.path.replace(variable, value)
             self.path = self.path + '.%s' % self.api.format
 
+        def _build_pagination_info(self, content_obj):
+            """Extract pagination information in the desired format."""
+            pagination = content_obj.get('pagination', {})
+            if self.pagination_format == 'next_url':
+                return pagination.get('next_url')
+            if self.pagination_format == 'dict':
+                return pagination
+            raise Exception('Invalid value for pagination_format: %s' % self.pagination_format)
+
         def _do_api_request(self, url, method="GET", body=None, headers=None):
             headers = headers or {}
             response, content = OAuth2Request(self.api).make_request(url, method=method, body=body, headers=headers)
             if response['status'] == '503':
                 raise InstagramAPIError(response['status'], "Rate limited", "Your client is making too many request per second")
-            content_obj = simplejson.loads(content)
+
+            try:
+                content_obj = simplejson.loads(content)
+            except ValueError:
+                raise InstagramClientError('Unable to parse response, not valid JSON.', status_code=response['status'])
+
+            # Handle OAuthRateLimitExceeded from Instagram's Nginx which uses different format to documented api responses
+            if not content_obj.has_key('meta'):
+                if content_obj.get('code') == 420:
+                    error_message = content_obj.get('error_message') or "Your client is making too many request per second"
+                    raise InstagramAPIError(420, "Rate limited", error_message)
+                raise InstagramAPIError(content_obj.has_key('code'), content_obj.has_key('error_type'), content_obj.has_key('error_message'))
+
             api_responses = []
             status_code = content_obj['meta']['code']
+            self.api.x_ratelimit_remaining = response.get("x-ratelimit-remaining",None)
+            self.api.x_ratelimit = response.get("x-ratelimit-limit",None)
             if status_code == 200:
                 if not self.objectify_response:
                     return content_obj, None
@@ -113,7 +146,7 @@ def bind_method(**config):
                         api_responses = self.root_class.object_from_dictionary(data)
                 elif self.response_type == 'empty':
                     pass
-                return api_responses, content_obj.get('pagination', {}).get('next_url')
+                return api_responses, self._build_pagination_info(content_obj)
             else:
                 raise InstagramAPIError(status_code, content_obj['meta']['error_type'], content_obj['meta']['error_message'])
 
@@ -126,11 +159,18 @@ def bind_method(**config):
                 yield api_responses, url
             return
 
+        def _get_with_next_url(self, url, method="GET", body=None, headers=None):
+            headers = headers or {}
+            content, next = self._do_api_request(url, method, body, headers)
+            return content, next
+
         def execute(self):
             url, method, body, headers = OAuth2Request(self.api).prepare_request(self.method,
                                                                                  self.path,
                                                                                  self.parameters,
                                                                                  include_secret=self.include_secret)
+            if self.with_next_url:
+                return self._get_with_next_url(self.with_next_url, method, body, headers)
             if self.as_generator:
                 return self._paginator_with_url(url, method, body, headers)
             else:
