@@ -17,6 +17,8 @@ class InstagramAPI(oauth2.OAuth2API):
     access_token_url = "https://api.instagram.com/oauth/access_token"
     protocol = "https"
     api_name = "Instagram"
+    x_ratelimit_remaining  = None
+    x_ratelimit = None
 
     def __init__(self, *args, **kwargs):
         format = kwargs.get('format', 'json')
@@ -33,7 +35,7 @@ class InstagramAPI(oauth2.OAuth2API):
 
     media_search = bind_method(
                 path="/media/search",
-                accepts_parameters=SEARCH_ACCEPT_PARAMETERS + ['lat', 'lng', 'min_timestamp', 'max_timestamp'],
+                accepts_parameters=SEARCH_ACCEPT_PARAMETERS + ['lat', 'lng', 'min_timestamp', 'max_timestamp', 'distance'],
                 root_class=Media)
 
     media_likes = bind_method(
@@ -128,7 +130,7 @@ class InstagramAPI(oauth2.OAuth2API):
 
     location_search = bind_method(
                 path="/locations/search",
-                accepts_parameters=SEARCH_ACCEPT_PARAMETERS + ['lat', 'lng', 'foursquare_id'],
+                accepts_parameters=SEARCH_ACCEPT_PARAMETERS + ['lat', 'lng', 'foursquare_id', 'foursquare_v2_id'],
                 root_class=Location)
 
     location = bind_method(
@@ -174,6 +176,15 @@ class InstagramAPI(oauth2.OAuth2API):
                 requires_target_user=True,
                 response_type="entry")
 
+    user_relationship = bind_method(
+                method="GET",
+                path="/users/{user_id}/relationship",
+                root_class=Relationship,
+                accepts_parameters=["user_id"],
+                paginates=False,
+                requires_target_user=True,
+                response_type="entry")
+
     def _make_relationship_shortcut(action):
         def _inner(self, *args, **kwargs):
             return self.change_user_relationship(user_id=kwargs.get("user_id"),
