@@ -6,6 +6,8 @@ class ApiModel(object):
     @classmethod
     def object_from_dictionary(cls, entry):
         # make dict keys all strings
+        if entry is None:
+            return ""
         entry_str_dict = dict([(str(key), value) for key, value in entry.items()])
         return cls(**entry_str_dict)
 
@@ -24,6 +26,12 @@ class Image(ApiModel):
         return "Image: %s" % self.url
 
 
+class Video(Image):
+
+    def __unicode__(self):
+        return "Video: %s" % self.url
+
+
 class Media(ApiModel):
 
     def __init__(self, id=None, **kwargs):
@@ -32,7 +40,21 @@ class Media(ApiModel):
             setattr(self, key, value)
 
     def get_standard_resolution_url(self):
-        return self.images['standard_resolution'].url
+        if self.type == 'image':
+            return self.images['standard_resolution'].url
+        else:
+            return self.videos['standard_resolution'].url
+
+    def get_low_resolution_url(self):
+        if self.type == 'image':
+            return self.images['low_resolution'].url
+        else:
+            return self.videos['low_resolution'].url
+
+
+    def get_thumbnail_url(self):
+        return self.images['thumbnail'].url
+
 
     def __unicode__(self):
         return "Media: %s" % self.id
@@ -40,12 +62,19 @@ class Media(ApiModel):
     @classmethod
     def object_from_dictionary(cls, entry):
         new_media = Media(id=entry['id'])
+        new_media.type = entry['type']
 
         new_media.user = User.object_from_dictionary(entry['user'])
+
         new_media.images = {}
         for version, version_info in entry['images'].iteritems():
             new_media.images[version] = Image.object_from_dictionary(version_info)
 
+        if new_media.type == 'video':
+            new_media.videos = {}
+            for version, version_info in entry['videos'].iteritems():
+                new_media.videos[version] = Video.object_from_dictionary(version_info)
+
         if 'user_has_liked' in entry:
             new_media.user_has_liked = entry['user_has_liked']
         new_media.like_count = entry['likes']['count']
